@@ -1,4 +1,4 @@
-/* $Id: file.c,v 1.21 2004/07/29 18:13:03 twogood Exp $ */
+/* $Id: file.c,v 1.23 2005/06/28 18:49:36 twogood Exp $ */
 #include "internal.h"
 #include "md5/global.h"
 #include "md5/md5.h"
@@ -68,6 +68,8 @@ static FileDescriptor* unshield_read_file_descriptor(Unshield* unshield, int ind
 
     case 6:
     case 7:
+    case 8:
+    case 9:
       saved_p = p = header->data +
           header->common.cab_descriptor_offset +
           header->cab.file_table_offset +
@@ -311,6 +313,8 @@ static bool unshield_reader_open_volume(UnshieldReader* reader, int volume)/*{{{
 
     case 6:
     case 7:
+    case 8:
+    case 9:
       {
         uint8_t six_header[VOLUME_HEADER_SIZE_V6];
         uint8_t* p = six_header;
@@ -548,6 +552,9 @@ static void unshield_reader_destroy(UnshieldReader* reader)/*{{{*/
 
 #define BUFFER_SIZE (64*1024+1)
 
+/*
+ * If filename is NULL, just throw away the result
+ */
 bool unshield_file_save (Unshield* unshield, int index, const char* filename)/*{{{*/
 {
   bool success = false;
@@ -562,7 +569,7 @@ bool unshield_file_save (Unshield* unshield, int index, const char* filename)/*{
 
 	MD5Init(&md5);
 
-  if (!unshield || !filename)
+  if (!unshield)
     goto exit;
 
   if (!(file_descriptor = unshield_get_file_descriptor(unshield, index)))
@@ -596,11 +603,14 @@ bool unshield_file_save (Unshield* unshield, int index, const char* filename)/*{
     goto exit;
   }
 
-  output = fopen(filename, "w");
-  if (!output)
+  if (filename) 
   {
-    unshield_error("Failed to open output file '%s'", filename);
-    goto exit;
+    output = fopen(filename, "w");
+    if (!output)
+    {
+      unshield_error("Failed to open output file '%s'", filename);
+      goto exit;
+    }
   }
 
   if (file_descriptor->flags & FILE_COMPRESSED)
@@ -670,10 +680,13 @@ bool unshield_file_save (Unshield* unshield, int index, const char* filename)/*{
 
     MD5Update(&md5, output_buffer, bytes_to_write);
 
-    if (bytes_to_write != fwrite(output_buffer, 1, bytes_to_write, output))
+    if (output)
     {
-      unshield_error("Failed to write %i bytes to file '%s'", bytes_to_write, filename);
-      goto exit;
+      if (bytes_to_write != fwrite(output_buffer, 1, bytes_to_write, output))
+      {
+        unshield_error("Failed to write %i bytes to file '%s'", bytes_to_write, filename);
+        goto exit;
+      }
     }
 
     total_written += bytes_to_write;
