@@ -1,7 +1,7 @@
-/* $Id: unshield.c,v 1.17 2004/07/29 18:18:40 twogood Exp $ */
+/* $Id: unshield.c,v 1.20 2005/07/08 14:08:37 twogood Exp $ */
 #define _BSD_SOURCE 1
 #define _POSIX_C_SOURCE 2
-#include <libunshield.h>
+#include "../lib/libunshield.h"
 #include <ctype.h>
 #include <locale.h>
 #include <stdio.h>
@@ -10,6 +10,13 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <unistd.h>
+#ifdef HAVE_CONFIG_H
+#include "../lib/unshield_config.h"
+#endif
+
+#ifndef VERSION
+#define VERSION "Unknown"
+#endif
 
 #define FREE(ptr)       { if (ptr) { free(ptr); ptr = NULL; } }
 
@@ -25,10 +32,13 @@ typedef enum
   ACTION_EXTRACT,
   ACTION_LIST_COMPONENTS,
   ACTION_LIST_FILE_GROUPS,
-  ACTION_LIST_FILES
+  ACTION_LIST_FILES,
+  ACTION_TEST
 } ACTION;
 
-static const char* output_directory   = ".";
+#define DEFAULT_OUTPUT_DIRECTORY  "."
+
+static const char* output_directory   = DEFAULT_OUTPUT_DIRECTORY;
 static const char* file_group_name    = NULL;
 static const char* component_name     = NULL;
 static bool junk_paths                = false;
@@ -37,6 +47,7 @@ static bool verbose                   = false;
 static ACTION action                  = ACTION_EXTRACT;
 static OVERWRITE overwrite            = OVERWRITE_ASK;
 static int log_level                  = UNSHIELD_LOG_LEVEL_LOWEST;
+static int exit_status                = 0;
 
 static bool make_sure_directory_exists(const char* directory)/*{{{*/
 {
@@ -87,7 +98,7 @@ static void show_usage(const char* name)
   fprintf(stderr,
       "Syntax:\n"
       "\n"
-      "\t%s [-c COMPONENT] [-d DIRECTORY] [-D LEVEL] [-g GROUP] [-G] [-h] [-l] c|l|x CABFILE\n"
+      "\t%s [-c COMPONENT] [-d DIRECTORY] [-D LEVEL] [-g GROUP] [-G] [-h] [-l] [-V] c|l|x CABFILE\n"
       "\n"
       "Options:\n"
       "\t-c COMPONENT  Only list/extract this component\n"
@@ -101,11 +112,13 @@ static void show_usage(const char* name)
       "\t-h            Show this help message\n"
       "\t-j            Junk paths (do not make directories)\n"
       "\t-L            Make file and directory names lowercase\n"
+      "\t-V            Print copyright and version information\n"
       "\n"
       "Commands:\n"
       "\tc             List components\n"         
       "\tg             List file groups\n"         
       "\tl             List files\n"
+      "\tt             Test files\n"
       "\tx             Extract files\n"
       "\n"
       "Other:\n"
@@ -127,7 +140,7 @@ static bool handle_parameters(
 {
 	int c;
 
-	while ((c = getopt(argc, argv, "c:d:D:g:hjLno")) != -1)
+	while ((c = getopt(argc, argv, "c:d:D:g:hjLnoV")) != -1)
 	{
 		switch (c)
     {
@@ -167,6 +180,11 @@ static bool handle_parameters(
         verbose = true;
         break;
 
+      case 'V':
+        printf("Unshield version " VERSION ". Copyright (C) 2003-2004 David Eriksson.\n");
+        exit(0);
+        break;
+
       case 'h':
       default:
         show_usage(argv[0]);
@@ -197,6 +215,10 @@ static bool handle_parameters(
       action = ACTION_LIST_FILES;
       break;
 
+    case 't':
+      action = ACTION_TEST;
+      break;
+
     case 'x':
       action = ACTION_EXTRACT;
       break;
@@ -291,6 +313,7 @@ static bool extract_file(Unshield* unshield, const char* prefix, int index)
         unshield_file_name(unshield, index),
         (log_level < 3) ? "Run unshield again with -D 3 for more information." : "");
     unlink(filename);
+    exit_status = 1;
   }
 
   return success;
@@ -310,6 +333,38 @@ static int extract_helper(Unshield* unshield, const char* prefix, int first, int
   return count;
 }/*}}}*/
 
+static bool test_file(Unshield* unshield, int index)
+{
+  bool success;
+
+  printf("  testing: %s\n", unshield_file_name(unshield, index));
+  success = unshield_file_save(unshield, index, NULL);
+
+  if (!success)
+  {
+    fprintf(stderr, "Failed to extract file '%s'.%s\n", 
+        unshield_file_name(unshield, index),
+        (log_level < 3) ? "Run unshield again with -D 3 for more information." : "");
+    exit_status = 1;
+  }
+
+  return success;
+}
+
+static int test_helper(Unshield* unshield, const char* prefix, int first, int last)/*{{{*/
+{
+  int i;
+  int count = 0;
+  
+  for (i = first; i <= last; i++)
+  {
+    if (unshield_file_is_valid(unshield, i) && test_file(unshield, i))
+      count++;
+  }
+
+  return count;
+}/*}}}*/
+
 static bool list_components(Unshield* unshield)
 {
   int i;
@@ -378,7 +433,7 @@ static int list_files_helper(Unshield* unshield, const char* prefix, int first,
       if (dirname[strlen(dirname)-1] != '/')
         strcat(dirname, "/");
 
-      printf(" %8i  %s%s\n",
+      printf(" %8" SIZE_FORMAT "  %s%s\n",
           unshield_file_size(unshield, i),
           dirname,
           unshield_file_name(unshield, i)); 
@@ -469,10 +524,20 @@ int main(int argc, char** argv)
     case ACTION_LIST_FILES:
       success = do_action(unshield, list_files_helper);
       break;
+      
+    case ACTION_TEST:
+      if (strcmp(output_directory, DEFAULT_OUTPUT_DIRECTORY) != 0)
+        fprintf(stderr, "Output directory (-d) option has no effect with test (t) command.\n");
+      if (make_lowercase)
+        fprintf(stderr, "Make lowercase (-L) option has no effect with test (t) command.\n");
+      success = do_action(unshield, test_helper);
+      break;
   }
 
 exit:
   unshield_close(unshield);
-  return success ? 0 : 1;
+  if (!success)
+    exit_status = 1;
+  return exit_status;
 }
 
